package com.psmreborn.nops1drm;

import android.content.ContentValues;
import android.content.Context;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import com.psmreborn.nopsmdrm.R;
import com.psmreborn.shared.Helper;
import com.psmreborn.shared.Logger;
import com.psmreborn.shared.Root;
import com.psmreborn.shared.StringEncryptor;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Locale;

public class PsmStartupCache {
    private static StringEncryptor stringEncryptor = null;
    private static String sha256HexStr(String str) {
        String lowerStr = str.toLowerCase();
        try {
            MessageDigest messageDigest = MessageDigest.getInstance("SHA-256");
            messageDigest.update(lowerStr.getBytes("UTF-8"));
            byte[] digest = messageDigest.digest();
            StringBuilder sb = new StringBuilder();
            for (byte hexDigit : digest) {
                sb.append(Integer.toHexString(hexDigit & 255));
            }
            return sb.toString();
        } catch (NoSuchAlgorithmException e) {
            return null;
        } catch (UnsupportedEncodingException e) {
            return null;
        }
    }
    private static void copyToPsm(Context ctx) throws Exception {
        String psmDataDir = Helper.getPsmAppInfo(ctx).dataDir;
        String psmDatabasesDir = new File(psmDataDir, "databases").getAbsolutePath();
        String psmStartContentDb = new File(psmDatabasesDir, "start_content.db").getAbsolutePath();

        String myDataDir = Helper.getAppInfo(ctx, ctx.getPackageName()).dataDir;
        String myDatabasesDir = new File(myDataDir, "databases").getAbsolutePath();
        String myStartContentDb = new File(myDatabasesDir, "start_content.db").getAbsolutePath();

        Root.mkdirAndChmodChown(psmDatabasesDir, 771, String.valueOf(Helper.getPsmUid(ctx)));
        Root.copyChmodAndChown(myStartContentDb, psmStartContentDb, 660, String.valueOf(Helper.getPsmUid(ctx)));
    }
    private static void copyFromPsm(Context ctx) throws Exception {
        String psmDataDir = Helper.getPsmAppInfo(ctx).dataDir;
        String psmDatabasesDir = new File(psmDataDir, "databases").getAbsolutePath();
        String psmStartContentDb = new File(psmDatabasesDir, "start_content.db").getAbsolutePath();

        String myDataDir = Helper.getAppInfo(ctx, ctx.getPackageName()).dataDir;
        String myDatabasesDir = new File(myDataDir, "databases").getAbsolutePath();
        String myStartContentDb = new File(myDatabasesDir, "start_content.db").getAbsolutePath();

        Root.mkdirAndChmodChown(myDatabasesDir, 771, String.valueOf(Helper.getMyUid(ctx)));
        Root.copyChmodAndChown(psmStartContentDb, myStartContentDb, 660, String.valueOf(Helper.getMyUid(ctx)));
    }
    private static int dbRemoveEntry(StartContentDb cdb, String entitlementId) {
        Logger.logText("DELETE, ENTITLEMENT_ID", entitlementId);
        SQLiteDatabase db = cdb.getWritableDatabase();
        int v = db.delete("cache_table", "entitlement_id = ?", new String[]{entitlementId});
        db.close();
        return v;
    }

    private static byte[] dbGetEntry(StartContentDb cdb, String entitlementId){
        Log.i("TicketGen", "dbGetEntry, "+entitlementId);
        SQLiteDatabase db = cdb.getReadableDatabase();
        Cursor query = db.query("cache_table", new String[] {"_id", "user_id", "entitlement_id", "ticket_data", "last_update"}, "entitlement_id = ?", new String[]{entitlementId}, null, null, null);
        if (query == null || query.getCount() != 1) {
            db.close();
            return null;
        } else {
            query.moveToFirst();
            byte[] ticketEnc = query.getBlob(query.getColumnIndex("ticket_data"));
            query.close();
            db.close();
            return stringEncryptor.decrypt(ticketEnc);
        }

    }
    private static void dbAddEntry(StartContentDb cdb, String userId, String entitlementId, byte[] ticket) throws Exception {
        Log.i("TicketGen", "dbAddEntry, "+entitlementId);

        SQLiteDatabase db = cdb.getWritableDatabase();
        ContentValues values = new ContentValues();

        byte[] encryptedTicket = stringEncryptor.encrypt(ticket);
        String shaUserId = sha256HexStr(userId);

        values.put("user_id", shaUserId);
        values.put("entitlement_id", entitlementId);
        values.put("ticket_data", encryptedTicket);
        values.put("last_update", System.currentTimeMillis());

        Logger.logText("USER_ID", shaUserId);
        Logger.logText("ENTITLEMENT_ID", entitlementId);
        Logger.logBytes("TICKET_DATA", ticket);
        Logger.logBytes("ENCRYPTED_TICKET_DATA", encryptedTicket);

        Long pos = db.insert("cache_table", null, values);

        db.close();

        if(pos <= -1)
            throw new Exception("failed to insert");
    }


    public static void addEntitlement(Context ctx, String entitlement_id) throws Exception {
        stringEncryptor = new StringEncryptor(Helper.getAndroidIdOfPsm(ctx), Helper.getPsmUid(ctx));
        try{copyFromPsm(ctx);} catch (Exception ignored) { };

        // get default values for onlineid, accountid, etc.
        String email = stringEncryptor.decryptString(Helper.getSharedPrefFromPsm(ctx, "SigninInfo", "SignedInUsername"), ctx.getResources().getString(R.string.default_email));
        Long accountId = Long.valueOf(stringEncryptor.decryptString(Helper.getSharedPrefFromPsm(ctx, "com.playstation.psstore_preferences", "last_signin_account_id"), String.valueOf(Helper.getDefaultAccountId(ctx))));
        String onlineId = ctx.getResources().getString(R.string.default_online_id);

        // read psm start content.db file ..
        StartContentDb scdb = new StartContentDb(ctx);
        byte[] ticket = dbGetEntry(scdb, entitlement_id);
        if(ticket == null){
            Log.i("TicketGen", "No ticket found, generating new!");
            dbAddEntry(scdb, email, entitlement_id, TicketGen.generateTicket(entitlement_id, Locale.getDefault().getLanguage(), onlineId, accountId));
        }
        else{
            if(!TicketGen.ticketExpired(ticket)) {
                Log.i("TicketGen", "Previous ticket expired, generating new one!");
                dbRemoveEntry(scdb, entitlement_id);
                dbAddEntry(scdb, email, entitlement_id, TicketGen.generateTicket(entitlement_id, Locale.getDefault().getLanguage(), onlineId, accountId));
            }
            else{
                Log.i("TicketGen", "Existing ticket is all good! continuing to PSM!");
            }
        }

        scdb.close();

        try{copyToPsm(ctx);} catch (Exception ignored) { };

    }
}
