package com.psmreborn.nops1drm;

import com.psmreborn.shared.Logger;

import java.util.Arrays;

public class TicketGen {

    private static final byte[] baseTicket = {(byte)0x41, (byte)0x00, (byte)0x00, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x01, (byte)0x5D, (byte)0x30, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x01, (byte)0x0F, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x14, (byte)0xBA, (byte)0x1D, (byte)0x80, (byte)0x73, (byte)0xC1, (byte)0xAE, (byte)0x02, (byte)0x75, (byte)0xD5, (byte)0xD3, (byte)0xED, (byte)0x8C, (byte)0x18, (byte)0x72, (byte)0x5A, (byte)0xA3, (byte)0x29, (byte)0x1C, (byte)0xC9, (byte)0xDF, (byte)0x00, (byte)0x01, (byte)0x00, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x01, (byte)0x00, (byte)0x00, (byte)0x07, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x00, (byte)0x01, (byte)0x65, (byte)0x95, (byte)0x4D, (byte)0xAC, (byte)0x7B, (byte)0x00, (byte)0x07, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x00, (byte)0x03, (byte)0xB9, (byte)0x03, (byte)0x74, (byte)0xCF, (byte)0x60, (byte)0x00, (byte)0x02, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x12, (byte)0x34, (byte)0x56, (byte)0x89, (byte)0xAB, (byte)0xCD, (byte)0xEF, (byte)0x00, (byte)0x04, (byte)0x00, (byte)0x20, (byte)0x4F, (byte)0x6E, (byte)0x6C, (byte)0x69, (byte)0x6E, (byte)0x65, (byte)0x49, (byte)0x64, (byte)0x68, (byte)0x65, (byte)0x72, (byte)0x65, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x04, (byte)0x65, (byte)0x6E, (byte)0x00, (byte)0x04, (byte)0x00, (byte)0x04, (byte)0x00, (byte)0x04, (byte)0x64, (byte)0x37, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x18, (byte)0x45, (byte)0x50, (byte)0x34, (byte)0x31, (byte)0x33, (byte)0x34, (byte)0x2D, (byte)0x4E, (byte)0x50, (byte)0x45, (byte)0x46, (byte)0x30, (byte)0x30, (byte)0x32, (byte)0x32, (byte)0x33, (byte)0x5F, (byte)0x30, (byte)0x30, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x30, (byte)0x11, (byte)0x00, (byte)0x04, (byte)0x07, (byte)0xC8, (byte)0x0C, (byte)0x06, (byte)0x00, (byte)0x01, (byte)0x00, (byte)0x04, (byte)0x19, (byte)0x00, (byte)0x02, (byte)0x00, (byte)0x30, (byte)0x10, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x0F, (byte)0x06, (byte)0x30, (byte)0x30, (byte)0x30, (byte)0x30, (byte)0x30, (byte)0x31, (byte)0x00, (byte)0x00, (byte)0x01, (byte)0x62, (byte)0x3F, (byte)0xCE, (byte)0x27, (byte)0xD0, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x40, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x30, (byte)0x02, (byte)0x00, (byte)0x44, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x04, (byte)0xBD, (byte)0x97, (byte)0x9F, (byte)0x07, (byte)0x00, (byte)0x08, (byte)0x00, (byte)0x38, (byte)0x30, (byte)0x35, (byte)0x02, (byte)0x18, (byte)0x08, (byte)0xA4, (byte)0xC8, (byte)0xD2, (byte)0x50, (byte)0x74, (byte)0xCC, (byte)0xFA, (byte)0xF2, (byte)0xBF, (byte)0xF8, (byte)0x49, (byte)0x27, (byte)0xC3, (byte)0x04, (byte)0xA7, (byte)0x79, (byte)0xE2, (byte)0x3C, (byte)0x30, (byte)0xCA, (byte)0xE6, (byte)0x2C, (byte)0xE5, (byte)0x02, (byte)0x19, (byte)0x00, (byte)0xE0, (byte)0x4C, (byte)0x84, (byte)0x9F, (byte)0x41, (byte)0x4D, (byte)0x6B, (byte)0x0E, (byte)0xFA, (byte)0xBE, (byte)0x38, (byte)0xC4, (byte)0x72, (byte)0x2A, (byte)0x11, (byte)0x46, (byte)0x8A, (byte)0x04, (byte)0x3D, (byte)0x50, (byte)0x68, (byte)0xDC, (byte)0x99, (byte)0xA7, (byte)0x00};
    private static byte[] longToBytes(long l) {
        byte[] result = new byte[8];
        for (int i = 8 - 1; i >= 0; i--) {
            result[i] = (byte)(l & 0xFF);
            l >>= Byte.SIZE;
        }
        return result;
    }
    private static long bytesToLong(final byte[] b) {
        long result = 0;
        for (int i = 0; i < 8; i++) {
            result <<= Byte.SIZE;
            result |= (b[i] & 0xFF);
        }
        return result;
    }
    private static void cpy(byte[] src, int srcOffset, byte[] dst, int dstOffset, int cpMaxLen){
        for(int i = 0; i < cpMaxLen; i++){
            byte b = 0;

            if((i + srcOffset) < src.length) {
                b = src[i + srcOffset];
            }

            dst[i + dstOffset] = b;
        }
    }

    public static boolean ticketExpired(byte[] ticket){
        byte[] expireDateBytes = new byte[0x8];
        cpy(ticket, 0x40, expireDateBytes, 0, 0x8);
        long expDate = bytesToLong(expireDateBytes);

        return expDate >= System.currentTimeMillis();
    }

    public static byte[] generateTicket(String entitlmentId, String language, String onlineId, Long accountId) throws Exception {
        Logger.logText("ENTITLEMENT_ID", entitlmentId);
        Logger.logText("LANGUAGE", language);
        Logger.logText("ONLINE_ID", onlineId);
        Logger.logText("ACCOUNT_ID", String.valueOf(accountId));

        long curTime = System.currentTimeMillis();
        long expireTime = System.currentTimeMillis() + (1209600 * 1000);

        Logger.logText("CUR_TIME", String.valueOf(curTime));
        Logger.logText("EXPIRE_TIME", String.valueOf(expireTime));

        String servicePart = entitlmentId.substring(0x0, 0x13);
        String entitlementPart = entitlmentId.substring(0x14);

        Logger.logText("SERVICE_PART", servicePart);
        Logger.logText("ENTITLEMENT_PART", entitlementPart);


        // get a copy of the base ticket ...
        byte[] newTicket = Arrays.copyOf(baseTicket, baseTicket.length);

        // copy strings to new ticket ..
        cpy(servicePart.getBytes("UTF-8"), 0, newTicket, 0x8C, 0x18);
        cpy(entitlementPart.getBytes("UTF-8"), 0, newTicket, 0xBB, 0x6);
        cpy(onlineId.getBytes("UTF-8"), 0, newTicket, 0x58, 0x20);
        cpy(language.getBytes("UTF-8"), 0, newTicket, 0x7C, 0x2);

        // copy longs to ticket
        cpy(longToBytes(accountId), 0, newTicket, 0x4C, 0x8);
        cpy(longToBytes(curTime), 0, newTicket, 0x34, 0x8);
        cpy(longToBytes(expireTime), 0, newTicket, 0x40, 0x8);

        // return the patched ticket.
        return newTicket;
    }
}
