package com.psmreborn.nopsmdrm;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.pm.PackageManager;
import android.os.Build;
import android.support.v4.app.ActivityCompat;

import java.util.ArrayList;

public class Permissions {
    public static boolean checkHavePermissions(Activity ctx, String[] permissions) {
        for(String permission : permissions){
            if (ActivityCompat.checkSelfPermission(ctx, permission) != PackageManager.PERMISSION_GRANTED) {
                return false;
            }
        }
        return true;
    }
    public static void requestPermissionsIfNeeded(Activity ctx, String[] permissions) {
        ArrayList<String> needPerms = new ArrayList<String>();
        for(String perm : permissions){
            if(ActivityCompat.checkSelfPermission(ctx, perm) != PackageManager.PERMISSION_GRANTED){
                needPerms.add(perm);
            }
        }
        if(!needPerms.isEmpty())
            ActivityCompat.requestPermissions(ctx, needPerms.toArray(new String[0]), 1);
    }
    public static boolean checkPermsAlert(Activity ctx){
        if(Build.VERSION.SDK_INT >= 23){
            if (!Permissions.checkHavePermissions(ctx, new String[] {
                    Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.KILL_BACKGROUND_PROCESSES,
                    Manifest.permission.CHANGE_WIFI_STATE,
                    Manifest.permission.ACCESS_WIFI_STATE,
                    Manifest.permission.INTERNET,
                    Manifest.permission.READ_PHONE_STATE,
            })) {
                new AlertDialog.Builder(ctx)
                        .setTitle("Missing permissions")
                        .setMessage("Missing required permissions, please grant all requested permissions\nPHONE is needed only to read IMEI for deriving \"ConsoleID\"\nSTORAGE is needed for backing up existing PSM data")
                        .setCancelable(false)
                        .setPositiveButton("Ok", new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialogInterface, int i) {
                                Permissions.requestPermissions(ctx);
                            }
                        }).show();

                return false;
            }
            else{
                return true;
            }
        }
        else{
            return true;
        }
    }

    public static void requestPermissions(Activity ctx) {
        if(Build.VERSION.SDK_INT >= 23){
            requestPermissionsIfNeeded(ctx, new String[] {
                    Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.KILL_BACKGROUND_PROCESSES,
                    Manifest.permission.CHANGE_WIFI_STATE,
                    Manifest.permission.ACCESS_WIFI_STATE,
                    Manifest.permission.INTERNET,
                    Manifest.permission.READ_PHONE_STATE,
            });
        }
    }
}
