package com.psmreborn.shared;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.ActivityManager;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Build;
import android.provider.Settings;

import com.psmreborn.nopsmdrm.R;
import com.rosstonovsky.abxutils.BinaryXmlPullParser;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class Helper {

    private static final String psmAppPackage = "com.playstation.psstore";
    public static boolean isNoPsmDrmAlreadyInstalled(Context ctx) {
        try {
            if(new File(getPsmAppInfo(ctx).nativeLibraryDir, "libdefault_real.so").exists()){
                return true;
            }
        } catch (PackageManager.NameNotFoundException e) {
            return false;
        }
        return false;
    }
    public static long getDefaultAccountId(Context ctx){
        return Long.valueOf(ctx.getResources().getString(R.string.default_account_id), 16);
    }
    public static String getSharedPrefFromPsm(Context ctx, String prefName, String prefKey) throws Exception {

        String psmSharedPrefs = new File(Helper.getPsmAppInfo(ctx).dataDir, "shared_prefs").getAbsolutePath();
        String psmPrefFile = new File(psmSharedPrefs, prefName+".xml").getAbsolutePath();

        String mySharedPrefs = new File(Helper.getAppInfo(ctx, ctx.getPackageName()).dataDir, "shared_prefs").getAbsolutePath();
        String myPrefFile = new File(mySharedPrefs, prefName+".xml").getAbsolutePath();

        Root.mkdirAndChmodChown(mySharedPrefs, 771, String.valueOf(Helper.getMyUid(ctx)));
        Root.copyChmodAndChown(psmPrefFile, myPrefFile, 660, String.valueOf(Helper.getMyUid(ctx)));

        SharedPreferences pref = ctx.getSharedPreferences(prefName, 0);
        return pref.getString(prefKey, null);
    }
    public static boolean isPsmInstalled(Context ctx){
        try {
            ctx.getPackageManager().getApplicationInfo(psmAppPackage, 0);
        } catch (PackageManager.NameNotFoundException e) {
            return false;
        }

        return true;
    }
    private static String parseSsaidText(FileInputStream stream) throws XmlPullParserException, IOException {
        XmlPullParserFactory factory = XmlPullParserFactory.newInstance();
        factory.setNamespaceAware(false);
        XmlPullParser xpp = factory.newPullParser();
        xpp.setInput(stream, "UTF-8");
        int eventType = xpp.getEventType();
        while (eventType != XmlPullParser.END_DOCUMENT) {
            if(eventType == XmlPullParser.START_TAG) {
                if(xpp.getName().equalsIgnoreCase("setting")){
                    String packageName = xpp.getAttributeValue(null,"package");
                    if(packageName != null && packageName.equalsIgnoreCase("com.playstation.psstore")){
                        return xpp.getAttributeValue(null,"value");
                    }
                }

            }
            eventType = xpp.next();
        }
        return null;
    }
    private static String parseSsaidBinary(FileInputStream stream) throws XmlPullParserException, IOException {
        // parse binary xml file ...
        BinaryXmlPullParser xpp = new BinaryXmlPullParser();
        xpp.setInput(stream, "UTF-8");
        int eventType = xpp.getEventType();
        while (eventType != BinaryXmlPullParser.END_DOCUMENT) {
            if(eventType == BinaryXmlPullParser.START_TAG) {
                if(xpp.getName().equalsIgnoreCase("setting")){
                    String packageName = xpp.getAttributeValue("package");
                    if(packageName != null && packageName.equalsIgnoreCase("com.playstation.psstore")){
                        return xpp.getAttributeValue("value");
                    }
                }

            }
            eventType = xpp.next();
        }
        return null;
    }

    @SuppressLint("HardwareIds")
    public static String getAndroidIdOfPsm(Context ctx) throws Exception {
        if(Build.VERSION.SDK_INT >= 26){
            String[] files = Root.findFiles("/data/system", "settings_ssaid.xml");
            File ssaidCacheFile = new File(ctx.getCacheDir(), "settings_ssaid.xml");

            for(String ssaidFile : files){
                Root.copyChmodAndChown(ssaidFile, ssaidCacheFile.getAbsolutePath(), 777, String.valueOf(Helper.getMyUid(ctx)));
                try{
                    FileInputStream stream = new FileInputStream(ssaidCacheFile);
                    return parseSsaidBinary(stream);
                }
                catch (XmlPullParserException e){
                    FileInputStream stream = new FileInputStream(ssaidCacheFile);
                    return parseSsaidText(stream);
                }

            }
            return null;
        }
        else {
            return Settings.Secure.getString(ctx.getContentResolver(), "android_id");
        }
    }

    public static int getMyUid(Context ctx) {
        try{
            ApplicationInfo myAppInfo = ctx.getPackageManager().getApplicationInfo(ctx.getPackageName(), 0);
            if(myAppInfo != null) {
                return myAppInfo.uid;
            }
        }
        catch (PackageManager.NameNotFoundException e) { };
        return 0;
    }
    public static int getPsmUid(Context ctx) {
        try{
            ApplicationInfo psmAppInfo = ctx.getPackageManager().getApplicationInfo(psmAppPackage, 0);
            if(psmAppInfo != null) {
                return psmAppInfo.uid;
            }
        }
        catch (PackageManager.NameNotFoundException e) { };
        return 0;
    }

    public static void killPsm(Context ctx){
        ActivityManager am = (ActivityManager) ctx.getSystemService(Activity.ACTIVITY_SERVICE);
        if (am != null) {
            am.killBackgroundProcesses(psmAppPackage);
        }
    }
    public static PackageInfo getPackageInfo(Context ctx, String pkg) throws PackageManager.NameNotFoundException {
        return ctx.getPackageManager().getPackageInfo(pkg, 0);
    }
    public static ApplicationInfo getAppInfo(Context ctx, String pkg) throws PackageManager.NameNotFoundException {
        return ctx.getPackageManager().getApplicationInfo(pkg, 0);
    }
    public static PackageInfo getPsmAppPkg(Context ctx) throws PackageManager.NameNotFoundException {
        return Helper.getPackageInfo(ctx,psmAppPackage);
    }
    public static ApplicationInfo getPsmAppInfo(Context ctx) throws PackageManager.NameNotFoundException {
        return Helper.getAppInfo(ctx,psmAppPackage);
    }
    @SuppressLint("SimpleDateFormat")
    public static String getDateTime(){
        return new SimpleDateFormat("MM_dd_yyyy_HH_mm_ss").format(new Date());
    }
}
