package com.psmreborn.shared;

import android.content.Context;
import android.os.Build;
import android.util.Log;

import com.psmreborn.nopsmdrm.R;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import eu.chainfire.libsuperuser.Shell;

public class Root {
    private static String busyboxBinary = null;

    public static void installMagiskModule(String magiskModuleFile) throws Exception {
        int res = 0;
        res = Shell.Pool.SU.run(new String[] { "magisk --install-module '"+magiskModuleFile+"'"});
        if(res != 0){
            throw new Exception("Magisk module failed to install.");
        }
    }
    public static void installRoot(String apkFile) throws Exception {
        ArrayList<String> results = new ArrayList<String>();
        int res = 0;

        String apkFileName = new File("/data/local/tmp", new File(apkFile).getName()).getAbsolutePath();
        moveChmodAndChown(apkFile, apkFileName, 777, null);

        if(Build.VERSION.SDK_INT >= 34){
            res = Shell.Pool.SU.run(new String[] { "pm install --bypass-low-target-sdk-block '"+apkFileName+"'"}, results, null, true);
        }
        else{
            res = Shell.Pool.SU.run(new String[] { "pm install '"+apkFileName+"'" },results, null, true);
        }

        // delete apk now its installed.
        deleteRoot(apkFileName);

        if(res != 0) {
            throw new Exception(results.get(0));
        }
    }
    public static void deleteRoot(String filename) throws Shell.ShellDiedException, IOException {
        Log.i("ROOT", "deleting file: "+filename);
        int res = Shell.Pool.SU.run(new String[] { busyboxBinary + " rm -f '"+filename+"'" });
        if(res != 0){
            throw new IOException("failed to remove file.");
        }
    }
    public static void reboot() throws Shell.ShellDiedException {
        Log.i("ROOT", "Rebooting");
        Shell.Pool.SU.run(new String[] { "reboot" });
    }
    public static boolean fileExistRoot(String filename) throws Shell.ShellDiedException {
        Log.i("ROOT", "FileExistRoot: " + filename);
        int res = Shell.Pool.SU.run(new String[] { busyboxBinary + " stat '" + filename +"'" });
        return res == 0;
    }

    public static String[] findFiles(String filePath, String fileName) throws Shell.ShellDiedException {
        ArrayList<String> results = new ArrayList<String>();
        int res = Shell.Pool.SU.run(new String[] { busyboxBinary + " find '" + filePath + "' -name '"+fileName+"'" }, results, null, true);
        if(res == 0) {
            String[] foundFiles = new String[results.size()];
            for(int i = 0; i < results.size(); i++)
                foundFiles[i] = results.get(i);

            return foundFiles;
        }
        return new String[]{};
    }

    public static void chownRoot(String filePath, String owner, String group) throws Shell.ShellDiedException, IOException {
        Log.i("ROOT", "ChownRoot: " + filePath);
        int res = Shell.Pool.SU.run(new String[] { busyboxBinary + " chown "+owner+":"+group+" -R '" + filePath +"'" });
        if(res != 0){
            Log.e("ROOT", "error (" + String.valueOf(res) +")");
            throw new IOException("Failed to chown:  "+filePath + " ("+String.valueOf(res)+")");
        }
    }
    public static String getFileGroup(String filePath) throws Shell.ShellDiedException, IOException {
        ArrayList<String> results = new ArrayList<String>();
        int res = Shell.Pool.SU.run(new String[] { busyboxBinary + " stat '" + filePath + "' -c %g" }, results, null, true);
        if(res == 0) {
            return results.get(0);
        }
        Log.e("ROOT", "error (" + String.valueOf(res) +")");
        throw new IOException("failed to get group: "+filePath);
    }
    public static String getFileOwner(String filePath) throws Shell.ShellDiedException, IOException {
        ArrayList<String> results = new ArrayList<String>();
        int res = Shell.Pool.SU.run(new String[] { busyboxBinary + " stat '" + filePath + "' -c %u" }, results, null, true);
        if(res == 0) {
            return results.get(0);
        }
        Log.e("ROOT", "error (" + String.valueOf(res) +")");
        throw new IOException("failed to get owner: "+filePath);
    }

    public static void tarRoot(ArrayList<String> src, String dst) throws IOException, Shell.ShellDiedException {
        if(src.size() <= 0){
            throw new IOException("tar src was empty.");
        };

        Log.i("ROOT", "TarRoot: "+ dst);

        StringBuilder tarFiles = new StringBuilder();
        for(String file : src) {
            tarFiles.append("'");
            tarFiles.append(file);
            tarFiles.append("' ");
        }

        int res = Shell.Pool.SU.run(new String[]{busyboxBinary + " tar -cf '" + dst + "' "+tarFiles.toString()});
        if (res != 0) {
            Log.e("ROOT", "error (" + String.valueOf(res) +")");
            throw new IOException("Failed to tar " + dst);
        }
    }
    public static void remountRo(String foldername) throws IOException, Shell.ShellDiedException {
        Log.i("ROOT", "Remounting as RO: " + foldername);
        int res = Shell.Pool.SU.run(new String[] { busyboxBinary +" mount -o ro,remount '"+foldername+"'"});
        if(res != 0){
            Log.e("ROOT", "error (" + String.valueOf(res) +")");
            throw new IOException("Failed to remmount:  "+foldername + " ("+String.valueOf(res)+")");
        }
    }
    public static void remountRw(String foldername) throws IOException, Shell.ShellDiedException {
        Log.i("ROOT", "Remounting as RW: " + foldername);
        int res = Shell.Pool.SU.run(new String[] { busyboxBinary +" mount -o rw,remount '"+foldername+"'"});
        if(res != 0){
            Log.e("ROOT", "error (" + String.valueOf(res) +")");
            throw new IOException("Failed to remmount:  "+foldername + " ("+String.valueOf(res)+")");
        }
    }
    public static void moveFileRoot(String filename, String destFilename) throws IOException, Shell.ShellDiedException {
        Log.i("ROOT", "MoveRoot: " + filename + " : "+ destFilename);
        int res = Shell.Pool.SU.run(new String[] { busyboxBinary +" mv '"+ filename + "' '" +destFilename +"'"});
        if(res != 0){
            Log.e("ROOT", "error (" + String.valueOf(res) +")");
            throw new IOException("Failed to rename "+filename+" to "+ destFilename);
        }
    }
    public static void mkdirAndChmodChown(String directory, int chmod, String chown) throws Shell.ShellDiedException {
        Log.i("ROOT", "MkdirAndChown: " + directory);
        Shell.Pool.SU.run(new String[]{
                busyboxBinary + " mkdir '" + directory +"'",
                busyboxBinary + " chmod " + String.valueOf(chmod) +" '"+directory+"'",
                busyboxBinary + " chown " + chown +":"+chown+" '"+directory+"'"
        });
    }

    public static void moveChmodAndChown(String srcFile, String dstFile, int chmod, String chown) throws IOException, Shell.ShellDiedException {
        Log.i("ROOT", "MoveAndChown: " + srcFile +" : "+dstFile);


        int chownRes = 0;
        int mvRes = 0;
        int chmodRes = 0;

        mvRes = Shell.Pool.SU.run(new String[]{busyboxBinary + " mv '" + srcFile +"' '"+dstFile+"'"});
        if(chmod != 0)
            chmodRes = Shell.Pool.SU.run(new String[]{busyboxBinary + " chmod " + String.valueOf(chmod) +" '"+dstFile+"'"});
        if(chown != null)
            chownRes = Shell.Pool.SU.run(new String[]{busyboxBinary + " chown " + chown +":"+chown+" '"+dstFile+"'"});

        if(mvRes != 0){
            Log.e("ROOT", "error (" + String.valueOf(mvRes) +")");
            throw new IOException("Failed to move file. " + String.valueOf(mvRes));
        }
        if(chmodRes != 0){
            Log.e("ROOT", "error (" + String.valueOf(chmodRes) +")");
            throw new IOException("Failed to chmod file. " + String.valueOf(chmodRes));
        }
        if(chownRes != 0){
            Log.e("ROOT", "error (" + String.valueOf(chownRes) +")");
            throw new IOException("Failed to chown file. " + String.valueOf(chownRes));
        }

    }
    public static void copyChmodAndChown(String srcFile, String dstFile, int chmod, String chown) throws IOException, Shell.ShellDiedException {
        Log.i("ROOT", "CopyAndChown: " + srcFile +" : "+dstFile);


        int chownRes = 0;
        int cpRes = 0;
        int chmodRes = 0;

        cpRes = Shell.Pool.SU.run(new String[]{busyboxBinary + " cp '" + srcFile +"' '"+dstFile+"'"});
        if(chmod != 0)
            chmodRes = Shell.Pool.SU.run(new String[]{busyboxBinary + " chmod " + String.valueOf(chmod) +" '"+dstFile+"'"});
        if(chown != null)
            chownRes = Shell.Pool.SU.run(new String[]{busyboxBinary + " chown " + chown +":"+chown+" '"+dstFile+"'"});

        if(cpRes != 0){
            Log.e("ROOT", "error (" + String.valueOf(cpRes) +")");
            throw new IOException("Failed to move file. " + String.valueOf(cpRes));
        }
        if(chmodRes != 0){
            Log.e("ROOT", "error (" + String.valueOf(chmodRes) +")");
            throw new IOException("Failed to chmod file. " + String.valueOf(chmodRes));
        }
        if(chownRes != 0){
            Log.e("ROOT", "error (" + String.valueOf(chownRes) +")");
            throw new IOException("Failed to chown file. " + String.valueOf(chownRes));
        }

    }
    public static void unpackResourceToLocationRoot(Context ctx, int resourceId, String outputDir, int chmod, String chown) throws IOException, Shell.ShellDiedException {
        Log.i("ROOT", "Unpacking resource and root copying to : " + outputDir);
        File tmpFile = File.createTempFile("tmp", "file", ctx.getCacheDir());
        tmpFile.createNewFile();

        Files.unpackResource(ctx, resourceId, tmpFile);
        moveChmodAndChown(tmpFile.getAbsolutePath(), outputDir, chmod, chown);
    }

    private static void setupBusyBox(Context ctx) throws IOException {
        Log.i("ROOT","Creating busybox binary");
        File tmpFile = new File(ctx.getCacheDir(), "busybox");
        if(!tmpFile.exists()) {
            tmpFile.createNewFile();

            if(tmpFile.setExecutable(true,false)) {
                Files.unpackResource(ctx, R.raw.busybox, tmpFile);
            }
            else {
                throw new IOException("failed to extract busybox binary.");
            }
        }

        busyboxBinary = tmpFile.getAbsolutePath();
    }

    public static boolean init(Context ctx) throws IOException {
        if(Shell.SU.available()){
            setupBusyBox(ctx);
            return true;
        }
        return false;
    }
}
