package com.psmreborn.shared;

import android.content.Context;
import android.util.Base64;
import android.util.Log;

import javax.crypto.*;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.SecretKeySpec;

import java.io.UnsupportedEncodingException;
import java.security.*;
import java.security.spec.InvalidKeySpecException;
import java.util.Arrays;


public class StringEncryptor {
    private static final byte[] iv = {-126, -30, -6, -75, -99, -117, -66, 117, 39, -65, -126, -27, -12, 38, -99, 86};
    private static final byte[] salt = {-92, -102, -105, -123, 71, -33, 69, -39, -27, -32, 21, 33, 126, -81, 69, 59, 57, 29, -83, -15};

    private String android_id = null;
    private int psm_uid = 0;

    public StringEncryptor(String androidId, int psmUid) {
        this.android_id = androidId;
        this.psm_uid = psmUid;
    }
    private String encodeBase64(byte[] data){
        return Base64.encodeToString(data, Base64.DEFAULT);
    }
    private byte[] decodeBase64(String str) {return Base64.decode(str, Base64.DEFAULT);}
    public String encryptString(String str) {
        byte[] data = str.getBytes();
        byte[] encryptedData = encrypt(data);
        if(encryptedData != null){
            byte[] encodedData = Arrays.copyOf(encryptedData, encryptedData.length + 2);
            encodedData[encodedData.length - 2] = 1;
            encodedData[encodedData.length - 1] = 1;

            return encodeBase64(encodedData);
        }
        return null;
    }

    public String decryptString(String str, String def) {
        if(str == null) return null;

        byte[] data = decodeBase64(str);
        byte[] arrayData = Arrays.copyOf(data, data.length - 2);

        byte[] decryptedData = decrypt(arrayData);
        if(decryptedData != null){
            try {
                return new String(decryptedData, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                Log.e("STRINGENCRYPTOR", e.toString());
                return def;
            }
        }
        return def;
    }
    public byte[] decrypt(byte[] input){
        if(input == null) return null;
        try {
            Cipher cipher = generateKeyCipher(StringEncryptor.salt, StringEncryptor.iv, Cipher.DECRYPT_MODE);
            if (cipher != null) {
                return cipher.doFinal(input);
            }
            else {
                Log.e("STRINGENCRYPTOR", "cipher was null");
            }
        } catch (BadPaddingException | IllegalBlockSizeException e) { System.out.println(e.toString()) ;}
        return null;

    }
    public byte[] encrypt(byte[] input){
        if(input == null) return null;
        try {
            Cipher cipher = generateKeyCipher(StringEncryptor.salt, StringEncryptor.iv, Cipher.ENCRYPT_MODE);
            if (cipher != null) {
                return cipher.doFinal(input);
            }
            else {
                Log.e("STRINGENCRYPTOR", "cipher was null");
            }
        } catch (BadPaddingException | IllegalBlockSizeException e) { System.out.println(e.toString()) ;}
        return null;

    }

    private Cipher generateKeyCipher(final byte[] salt, final byte[] iv, final int opmode) {
        try {
            final String androidId = this.android_id;
            final String psmUid = String.valueOf(this.psm_uid);
            if (androidId == null || psmUid == null) {
                throw new InvalidParameterException();
            }
            final SecretKeyFactory skeyFactory = SecretKeyFactory.getInstance("PBEWITHSHAAND256BITAES-CBC-BC");
            final char[] charArray = (androidId + psmUid + "     com.playstation.psstore    ").toCharArray();
            final PBEKeySpec keySpec = new PBEKeySpec(charArray, salt, 16, 256);
            Arrays.fill(charArray, '\0');
            final SecretKeySpec key = new SecretKeySpec(skeyFactory.generateSecret(keySpec).getEncoded(), "AES");
            final Cipher newCipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
            newCipher.init(opmode, key, new IvParameterSpec(iv), new SecureRandom());
            return newCipher;
        }
        catch (NoSuchPaddingException ex) {
            Log.e("STRINGENCRYPTOR", ex.toString());
            return null;
        }
        catch (InvalidAlgorithmParameterException ex2) {
            Log.e("STRINGENCRYPTOR", ex2.toString());
            return null;
        }
        catch (InvalidKeyException ex3) {
            Log.e("STRINGENCRYPTOR", ex3.toString());
            return null;
        }
        catch (InvalidKeySpecException ex4) {
            Log.e("STRINGENCRYPTOR", ex4.toString());
            return null;
        }
        catch (NoSuchAlgorithmException ex5) {
            Log.e("STRINGENCRYPTOR", ex5.toString());
            return null;
        }
    }
}
